<?php

require_once "abstract.ilPluginControllerGUI.php";

/**
 * Class ilPluginControllerFormGUI
 *
 * Date: 02.10.13
 * Time: 15:10
 * @author Thomas Joußen <tjoussen@databay.de>
 */
abstract class ilPluginControllerFormGUI extends ilPluginControllerGUI
{

    /**
     * @var ilPropertyFormGUI
     */
    protected $form;

    public function edit()
    {
        $model = $this->loadModel();
        $this->initForm($model);
        $this->bindModelToForm($model);
        $this->tpl->setContent($this->form->getHTML());
    }

    public function update()
    {
        $model = $this->loadModel();
        $this->initForm($model);
        if ($this->form->checkInput()) {
            $model->bindForm($this->form);
            $model->update();
            ilUtil::sendSuccess($this->plugin->txt("msg_obj_modified"), true);
            $this->redirect(
                $this->getAfterUpdateRedirectCmd(),
                $this->getAfterUpdateRedirectParams($model)
            );
        }

        $this->form->setValuesByPost();
        $this->tpl->setContent($this->form->getHTML());
    }

    /**
     * @param ilPluginModelInterface $model
     */
    protected function bindModelToForm(ilPluginModelInterface $model)
    {
        $this->form->setValuesByArray($model->toArray());
    }

    /**
     * Loads and returns the Model for the controller gui
     *
     * @return ilPluginModelInterface
     */
    abstract protected function loadModel();

    /**
     * Initialize the PropertyFormGUI for this controller
     *
     * @param ilPluginModel $model
     *
     * @return mixed
     */
    abstract protected function initForm($model);

    /**
     * Get the redirect action after an update was successful
     *
     * @return string
     */
    abstract protected function getAfterUpdateRedirectCmd();

    /**
     * @param $model
     *
     * @return array
     */
    abstract protected function getAfterUpdateRedirectParams($model);
}
