<?php

require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/class.ilObjQuestionSetPoolGUI.php";

/**
 * Class ilPluginControllerGUI
 *
 * Date: 23.09.13
 * Time: 14:10
 * @author Thomas Joußen <tjoussen@databay.de>
 */
abstract class ilPluginControllerGUI
{

    /**
     * The main Controller of the Plugin
     *
     * @var ilObjQuestionSetPoolGUI
     */
    public $controller;

    /**
     * @var ilCtrl
     */
    public $ctrl;

    /**
     * @var ilToolbarGUI;
     */
    protected $toolbar;

    /**
     * @var ilTemplate
     */
    protected $tpl;

    /**
     * @var ilPlugin
     */
    protected $plugin;

    /**
     * @var ilLanguage
     */
    protected $lng;

    /**
     * @var ilTabsGUI
     */
    protected $tabs;

    /**
     * @var ilObjUser
     */
    protected $user;

    /** @var \ILIAS\UI\Factory */
    protected $uiFactory;
    /** @var \ILIAS\UI\Renderer */
    protected $uiRenderer;

    /**
     * @param ilObjQuestionSetPoolGUI $controller
     */
    public function __construct(ilObjQuestionSetPoolGUI $controller)
    {
        global $DIC;

        $this->controller = $controller;
        $this->ctrl = $DIC->ctrl();
        $this->tpl = $DIC->ui()->mainTemplate();
        $this->plugin = $this->controller->plugin;
        $this->lng = $DIC->language();
        $this->toolbar = $DIC->toolbar();
        $this->tabs = $DIC->tabs();
        $this->user = $DIC->user();
        $this->uiFactory = $DIC->ui()->factory();
        $this->uiRenderer = $DIC->ui()->renderer();

        $this->activateTab();
        $this->includeRequirements();
    }

    protected function initToolbar()
    {
        $this->toolbar->setFormAction($this->ctrl->getFormAction($this->controller));
    }

    protected function addToolbarButton($text, $command, array $params = array())
    {
        if ($this->toolbar == null) {
            $this->initToolbar();
        }

        foreach ($params as $key => $param) {
            $this->ctrl->setParameter($this->controller, $key, $param);
        }

        $link = $this->ctrl->getLinkTarget($this->controller, $command);

        foreach ($params as $key => $param) {
            $this->ctrl->setParameter($this->controller, $key, "");
        }

        $this->toolbar->addButton($this->plugin->txt($text), $link);
    }

    /**
     * Activates the tab for the current GUI
     */
    private function activateTab()
    {
        $this->tabs->activateTab($this->getCurrentTab());
    }

    private function includeRequirements()
    {
        $requirements = $this->getRequirements();

        if (!empty($requirements)) {
            foreach ($requirements as $requirement) {
                require_once $this->controller->plugin->getDirectory() . DIRECTORY_SEPARATOR . "classes" . DIRECTORY_SEPARATOR . $requirement;
            }
        }
    }

    protected function redirect($command, $params = array())
    {
        foreach ($params as $key => $param) {
            $this->ctrl->setParameter($this->controller, $key, $param);
        }

        $this->ctrl->redirect($this->controller, $command);
    }

    /**
     * Returns the current tab name
     *
     * return string
     */
    abstract protected function getCurrentTab();

    /**
     * @return array
     */
    abstract protected function getRequirements();
}
