<?php

require_once "abstract.ilPluginControllerTableGUI.php";

/**
 * Class ilQuestionSetPoolSetQuestionsGUI
 *
 * Date: 08.10.13
 * Time: 11:11
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilQuestionSetPoolSetQuestionsGUI extends ilPluginControllerTableGUI
{
    public function __construct(ilObjQuestionSetPoolGUI $controller)
    {
        parent::__construct($controller);
        $this->addSubTabs();
    }

    /**
     *
     */
    public function listQuestions()
    {
        $this->controller->ensurePermission('write');

        if ($this->controller->object instanceof ilObjQuestionSetPool) {
            $this->addToolbarButton("back_to_set_list", "ilQuestionSetPoolSetsGUI.show");
        }
        $this->addToolbarButton("browse_for_questions", "ilQuestionSetPoolSetQuestionsGUI.browseForQuestions", array(
            "set_id" => (int) $_GET['set_id']
        ));

        $set = new ilQuestionSetPoolSet((int) $_GET['set_id']);
        $set->read();

        $this->ctrl->setParameter($this->controller, "set_id", (int) $_GET["set_id"]);
        $table = new ilQuestionSetPoolSetQuestionsTableGUI($this->controller, $this->plugin, "ilQuestionSetPoolSetQuestionsGUI.listQuestions", ilQuestionSetPoolSetQuestionsTableGUI::CONTEXT_LIST_SET_QST, (int) $_GET['set_id']);
        $table->setModelList($set->getQuestionList());

        $table->setTitle(
            $set->getTitle() . " - " .
            $this->plugin->txt("questions")
        );
        $table->addMultiCommand("ilQuestionSetPoolSetQuestionsGUI.removeMultipleConfirmation", $this->controller->lng->txt("remove"));
        $table->addMultiCommand("ilQuestionSetPoolSetQuestionsGUI.copyQuestion", $this->controller->lng->txt("duplicate"));
        $table->populate();

        $this->tpl->setContent($table->getHTML());
    }

    /**
     *
     */
    public function browseForQuestions()
    {
        $this->controller->ensurePermission('write');

        $this->ctrl->setParameter($this->controller, "set_id", (int) $_GET['set_id']);

        $this->addToolbarButton("back_to_qst_list", __CLASS__ . ".listQuestions");
        
        $table = new ilQuestionSetPoolQuestionsTableGUI($this->controller, $this->plugin, "ilQuestionSetPoolSetQuestionsGUI.browseForQuestions", ilQuestionSetPoolQuestionsTableGUI::CONTEXT_LIST_LINKED_QST_IN_SET, (int) $_GET['set_id']);
        $table->setGuiClassName("ilQuestionSetPoolSetQuestionsGUI");
        $table->initFilter();
        $modellist = new ilQuestionPoolQuestionList();
        $modellist->setQuestionSetPool($this->controller->object);
        $table->setModelList($modellist);
        $table->addMultiCommand("ilQuestionSetPoolSetQuestionsGUI.insertQuestions", $this->controller->lng->txt("insert"));
        $table->populate();

        ilUtil::sendInfo($this->plugin->txt("qs_question_info"), true);

        $this->tpl->setContent($table->getHTML());
    }

    /**
     *
     */
    public function insertQuestions()
    {
        $this->controller->ensurePermission('write');
        
        if (!$this->dataSelected("q_id")) {
            ilUtil::sendInfo($this->plugin->txt("no_questions_selected"), true);
            $this->redirect("ilQuestionSetPoolSetQuestionsGUI.browseForQuestions", array(
                'set_id' => (int) $_GET['set_id']
            ));
        }

        $set = new ilQuestionSetPoolSet((int) $_GET["set_id"]);
        $set->read();
        $success = $set->getQuestionList()->insert($_POST['q_id'], $this->controller->object);

        if (!$success) {
            ilUtil::sendFailure($this->plugin->txt("questions_added_error"), true);
            $this->redirect("ilQuestionSetPoolSetQuestionsGUI.browseForQuestions", array(
                    'set_id' => (int) $_GET['set_id']
                ));
        }

        ilUtil::sendSuccess($this->plugin->txt("questions_added"), true);
        $this->redirect("ilQuestionSetPoolSetQuestionsGUI.listQuestions", array(
                'set_id' => (int) $_GET['set_id']
            ));
    }

    /**
     * Called if a single set should be deleted, calls \\ilQuestionSetPoolSetQuestionsGUI::removeMultipleConfirmation do show the confirmation screen
     */
    public function removeConfirmation()
    {
        if (!isset($_GET['q_id'])) {
            ilUtil::sendInfo($this->plugin->txt('no_questions_selected_to_delete'));
            $this->listQuestions();
            return;
        }

        $_POST['q_id'] = array((int) $_GET['q_id']);
        $this->removeMultipleConfirmation();
    }

    /**
     *
     */
    public function copyQuestion()
    {
        $this->controller->ensurePermission('write');

        if (!isset($_POST['q_id']) || !is_array($_POST['q_id']) || !count($_POST['q_id'])) {
            ilUtil::sendFailure($this->lng->txt('copy_no_questions_selected'));
            $this->listQuestions();
        }

        $question_ids = array_filter((array) $_POST['q_id']);

        $set = new ilQuestionSetPoolSet((int) $_GET["set_id"]);
        $set->read();
        $set->getQuestionList()->insert($question_ids, $this->controller->object);
        if (is_array($question_ids)) {
            if (1 == count($question_ids)) {
                ilUtil::sendSuccess($this->plugin->txt('copy_questions_success_s'), true);
            } else {
                ilUtil::sendSuccess($this->plugin->txt('copy_questions_success_p'), true);
            }
            $this->redirect("ilQuestionSetPoolSetQuestionsGUI.listQuestions", array(
                'set_id' => (int) $_GET['set_id']
            ));
        }
    }

    /**
     *
     */
    public function removeMultipleConfirmation()
    {
        $this->controller->ensurePermission('write');

        if (!$this->dataSelected('q_id')) {
            ilUtil::sendInfo($this->plugin->txt('no_questions_selected_to_delete'));
            $this->listQuestions();
            return;
        }

        $this->redirectIfQuestionIsUsedInPath();

        $confirmation = new ilConfirmationGUI();
        $this->ctrl->setParameter($this->controller, 'set_id', (int) $_GET['set_id']);
        $confirmation->setFormAction($this->ctrl->getFormAction($this->controller, __CLASS__ . '.listQuestions'));
        $this->ctrl->setParameter($this->controller, 'set_id', '');
        $confirmation->setCancel($this->lng->txt('cancel'), __CLASS__ . '.listQuestions');
        $confirmation->setConfirm($this->lng->txt('confirm'), __CLASS__ . '.removeQuestions');

        $num_items = 0;
        foreach ($_POST['q_id'] as $question_id) {
            $question = assQuestion::_instanciateQuestion($question_id);
            $confirmation->addItem('q_id[]', $question_id, $question->getTitle());
            ++$num_items;
        }

        if (1 == $num_items) {
            $confirmation->setHeaderText($this->plugin->txt('remove_selected_questions_s'));
        } else {
            $confirmation->setHeaderText($this->plugin->txt('remove_selected_questions_p'));
        }

        $this->tpl->setContent($confirmation->getHTML());
    }

    public function removeQuestions()
    {
        $this->controller->ensurePermission('write');
        
        if (!$this->dataSelected('q_id')) {
            ilUtil::sendInfo($this->plugin->txt('no_questions_selected_to_delete'), true);
            $this->redirect('ilQuestionSetPoolSetQuestionsGUI.listQuestions', array(
                'set_id' => (int) $_GET['set_id']
            ));
        }

        $this->redirectIfQuestionIsUsedInPath();

        $questions_to_delete = (array) $_POST['q_id'];

        $model = new ilQuestionSetPoolSet((int) $_GET['set_id']);
        $model->read();
        $model->getQuestionList()->remove($questions_to_delete);

        if (is_array($questions_to_delete) && count($questions_to_delete) == 1) {
            ilUtil::sendSuccess($this->plugin->txt('deleted_selected_questions_s'), true);
        } else {
            ilUtil::sendSuccess($this->plugin->txt('deleted_selected_questions_p'), true);
        }
        $this->redirect('ilQuestionSetPoolSetQuestionsGUI.listQuestions', array(
            'set_id' => (int) $_GET['set_id']
        ));
    }

    /**
     *
     */
    public function showQuestionPreview()
    {
        $this->ctrl->setParameter($this->controller, "set_id", (int) $_GET['set_id']);
        if (isset($_GET['referer']) && 'browse' == $_GET['referer']) {
            $this->toolbar->addButton($this->plugin->txt('back_to_qst_list'), $this->ctrl->getLinkTarget($this->controller, __CLASS__ . '.browseForQuestions'));
        } else {
            $this->toolbar->addButton($this->plugin->txt('back_to_qst_list'), $this->ctrl->getLinkTarget($this->controller, __CLASS__ . '.listQuestions'));
        }
        $this->ctrl->setParameter($this->controller, "set_id", "");

        $this->controller->populateContentStyleBlock();
        $this->controller->populateSyntaxStyleBlock();

        $gui = assQuestion::instantiateQuestionGUI((int) $_GET['q_id']);
        $gui->setTargetGuiClass('ilObjQuestionSetPoolGUI');
        $preview = $gui->getPreview();
        $this->tpl->setContent($preview);
    }

    public function applyAvailableFilter()
    {
        $table = new ilQuestionSetPoolQuestionsTableGUI($this->controller, $this->plugin, "ilQuestionSetPoolSetQuestionsGUI.browseForQuestions", ilQuestionSetPoolQuestionsTableGUI::CONTEXT_LIST_LINKED_QST_IN_SET, (int) $_GET['set_id']);
        $table->setGuiClassName("ilQuestionSetPoolSetQuestionsGUI");
        $table->initFilter();
        $table->resetOffset();
        $table->writeFilterToSession();

        $this->browseForQuestions();
    }

    public function resetAvailableFilter()
    {
        $table = new ilQuestionSetPoolQuestionsTableGUI($this->controller, $this->plugin, "ilQuestionSetPoolSetQuestionsGUI.browseForQuestions", ilQuestionSetPoolQuestionsTableGUI::CONTEXT_LIST_LINKED_QST_IN_SET, (int) $_GET['set_id']);
        $table->setGuiClassName("ilQuestionSetPoolSetQuestionsGUI");
        $table->initFilter();
        $table->resetOffset();
        $table->resetFilter();

        $this->browseForQuestions();
    }

    /**
     * Returns the current tab name
     * return string
     */
    protected function getCurrentTab()
    {
        return "content";
    }

    /**
     * @return array
     */
    protected function getRequirements()
    {
        return array(
            "tables/class.ilQuestionSetPoolQuestionsTableGUI.php",
            "tables/class.ilQuestionSetPoolSetQuestionsTableGUI.php",
            "models/class.ilQuestionSetPoolSet.php",
            "models/class.ilQuestionSetPoolPath.php",
            "models/class.ilQuestionPoolQuestionList.php",

        );
    }

    /**
     *
     */
    public function editQuestion()
    {
        $this->controller->ensurePermission('write');
        
        if (!isset($_GET['q_id']) || !(int) $_GET['q_id']) {
            $this->listQuestions();
            return;
        }
        
        $params = array(
            'set_id' => (int) $_GET['set_id'],
            'cmd' => __CLASS__ . '.listQuestions'
        );

        $question_id = (int) $_GET['q_id'];
        ilUtil::redirect('ilias.php?consumer_context=' . base64_encode(json_encode($params)) . '&baseClass=ilObjQuestionPoolGUI&ref_id=' . $this->controller->object->getRefId() . '&cmd=editQuestionForTest&calling_consumer=' . $this->controller->object->getRefId() . '&q_id=' . $question_id);
    }

    private function addSubTabs()
    {
        $this->ctrl->setParameter($this->controller, "set_id", (int) $_GET['set_id']);

        $this->tabs->addSubTab(
            "set_questions",
            $this->plugin->txt("questions"),
            $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolSetQuestionsGUI.listQuestions")
        );

        $this->tabs->addSubTab(
            "paths",
            $this->plugin->txt("paths"),
            $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathsGUI.listPaths")
        );

        $this->tabs->addSubTab(
            "path_overview",
            $this->plugin->txt("path_overview"),
            $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolPathGraphGUI.showPathOverview")
        );

        $this->tabs->activateSubTab("set_questions");

        $this->ctrl->setParameter($this->controller, "set_id", "");
    }

    /**
     *
     */
    protected function redirectIfQuestionIsUsedInPath()
    {
        $model = new ilQuestionSetPoolSet((int) $_GET['set_id']);
        $model->read();
        $paths = $model->getPathList()->getData();
        
        foreach ((array) $_POST['q_id'] as $question_id) {
            foreach ($paths as $path_data) {
                $path = new ilQuestionSetPoolPath($path_data['id']);
                $path->read();

                if ($path->isQuestionInPath($question_id)) {
                    ilUtil::sendFailure($this->plugin->txt('questions_used_in_path'), true);
                    $this->redirect(
                        'ilQuestionSetPoolSetQuestionsGUI.listQuestions',
                        array(
                            'set_id' => (int) $_GET['set_id']
                        )
                    );
                }
            }
        }
    }
}
