<?php

require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/models/abstract.ilPluginModelList.php";


/**
 * Class ilPluginTableGUI
 *
 * Date: 24.09.13
 * Time: 08:59
 * @author Thomas Joußen <tjoussen@databay.de>
 */
abstract class ilPluginTableGUI extends ilTable2GUI
{

    /**
     * @var ilPluginModelList
     */
    protected $model_list;

    /**
     * @var ilCtrl
     */
    protected $ctrl;

    /**
     * @var array
     */
    protected $visibleOptionalColumns = array();

    /**
     * @var array
     */
    protected $optionalColumns = array();

    /**
     * @var array
     */
    protected $filter = array();

    /**
     * @var array
     */
    protected $optional_filter = array();

    /**
     * @var
     */
    protected $plugin;

    public function __construct($a_parent_obj, $a_parent_cmd = "", $plugin)
    {
        $this->plugin = $plugin;
        parent::__construct($a_parent_obj, $a_parent_cmd);

        $this->lng->loadLanguageModule("assessment");
    }


    /**
     * @param ilPluginModelList $model_list
     */
    public function setModelList($model_list)
    {
        $this->model_list = $model_list;
    }

    /**
     * @return ilPluginModelList
     */
    public function getModelList()
    {
        return $this->model_list;
    }

    /**
     * @param string $column
     * @return bool
     */
    protected function isColumnVisible($column)
    {
        if (array_key_exists($column, $this->optionalColumns) && !isset($this->visibleOptionalColumns[$column])) {
            return false;
        }

        return true;
    }

    /**
     * This method can be used to prepare values for sorting (e.g. translations), to filter items etc.
     * It is called before sorting and segmentation.
     * @param array $data
     * @return array
     */
    protected function prepareData(array &$data)
    {
        foreach ($data as $key => $item) {
            if (!is_array($item)) {
                $data[$key] = $item->toArray();
            }
        }
    }

    /**
     * This method can be used to manipulate the data of a row after sorting and segmentation
     * @param array $data
     * @return array
     */
    protected function prepareRow(array &$row)
    {
    }

    /**
     * Define a final formatting for a cell value
     * @param mixed  $column
     * @param array  $row
     * @return mixed
     */
    protected function formatCellValue($column, array $row)
    {
        return $row[$column];
    }

    /**
     * @param array $row
     */
    final protected function fillRow($row)
    {
        $this->prepareRow($row);

        foreach ($this->getStaticData() as $column) {
            $value = $this->formatCellValue($column, $row);
            $this->tpl->setVariable('VAL_' . strtoupper($column), $value);
        }

        foreach ($this->optionalColumns as $index => $definition) {
            if (!$this->isColumnVisible($index)) {
                continue;
            }

            $this->tpl->setCurrentBlock('optional_column');
            $value = $this->formatCellValue($index, $row);
            if ((string) $value === '') {
                $this->tpl->touchBlock('optional_column');
            } else {
                $this->tpl->setVariable('OPTIONAL_COLUMN_VAL_' . strtoupper($index), $value);
            }

            $this->tpl->parseCurrentBlock();
        }
    }

    /**
     * Return an array of all static (always visible) data fields in a row.
     * For each key there has to be a variable name VAL_<COLUMN_KEY> in your defined row template.
     * Example:
     *     return array('title', 'checkbox');
     *     There have to be two template variables: VAL_TITLE and VAL_CHECKBOX
     * @return array
     * @abstract
     */
    abstract protected function getStaticData();

    /**
     * @throws ilException
     */
    public function populate()
    {
        /*$data = $this->getModelList()->getData();
        if($data != null)
        {
            $this->prepareData($data);
        }

        $this->setData($data);*/
        /**
         * @TODO MOVE TO A PROVIDER!
         */
        if (!$this->getExternalSegmentation() && $this->getExternalSorting()) {
            $this->determineOffsetAndOrder(true);
        } elseif ($this->getExternalSegmentation() || $this->getExternalSorting()) {
            $this->determineOffsetAndOrder();
        }
        $params = array();
        if ($this->getExternalSegmentation()) {
            $params['limit'] = $this->getLimit();
            $params['offset'] = $this->getOffset();
        }
        if ($this->getExternalSorting()) {
            $params['order_field'] = $this->getOrderField();
            $params['order_direction'] = $this->getOrderDirection();
        }

        $this->determineSelectedFilters();
        $filter = $this->filter;

        foreach ($this->optional_filter as $key => $value) {
            if ($this->isFilterSelected($key)) {
                $filter[$key] = $value;
            }
        }

        $data = $this->getModelList()->loadData($params, $filter);

        if (is_array($data['items'])) {
            if (!count($data['items']) && $this->getOffset() > 0 && $this->getExternalSegmentation()) {
                $this->resetOffset();
                $params['limit'] = $this->getLimit();
                $params['offset'] = $this->getOffset();
                $data = $this->getModelList()->loadData($params, $filter);
            }
        }

        if ($data != null) {
            $this->prepareData($data);
        }

        $this->setData($data);
        if ($this->getExternalSegmentation()) {
            if (is_array($data)) {
                $this->setMaxCount(count($data));
            }
        }
    }
}
