<?php

require_once "class.ilPluginTableGUI.php";

/**
 * Class ilQuestionSetPoolSetsTableGUI
 *
 * Date: 07.10.13
 * Time: 13:49
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ilQuestionSetPoolSetsTableGUI extends ilPluginTableGUI
{
    public function __construct($controller, $plugin)
    {
        global $DIC;

        $this->ctrl = $DIC->ctrl();
        $this->setId("xqsp_sets_" . $plugin->getId());

        parent::__construct($controller, "", $plugin);

        $this->createTable();
    }

    private function createTable()
    {
        $this->setTitle($this->plugin->txt("setsbrowser"));

        $this->setFormName('setsbrowser');
        $this->setStyle('table', 'fullwidth');
        $this->addColumn('', '', '1%', true);

        $this->addColumn($this->lng->txt("title"), 'title', '');
        $this->addColumn($this->lng->txt("description"), 'description', '');
        $this->addColumn($this->lng->txt("actions"), '', '1%');

        $this->setSelectAllCheckbox('set_id');

        $this->setRowTemplate("tpl.il_xqsp_qs_row.html", $this->parent_obj->plugin->getDirectory());

        $this->setFormAction($this->ctrl->getFormAction($this->parent_obj));
        $this->setDefaultOrderField("title");
        $this->setDefaultOrderDirection("asc");

        $this->addMultiCommand('ilQuestionSetPoolSetsGUI.removeMultipleConfirmation', $this->lng->txt('delete'));

        $this->enable('sort');
        $this->enable('header');
        $this->enable('select_all');
    }

    /**
     * Return an array of all static (always visible) data fields in a row.
     * For each key there has to be a variable name VAL_<COLUMN_KEY> in your defined row template.
     * Example:
     *     return array('title', 'checkbox');
     *     There have to be two template variables: VAL_TITLE and VAL_CHECKBOX
     * @return array
     */
    protected function getStaticData()
    {
        return array(
            'checkbox',
            'title',
            'title_link',
            "description",
            "author",
            "points",
            "tstamp",
            "created",
            "actions"
        );
    }

    /**
     * Define a final formatting for a cell value
     *
     * @param mixed $column
     * @param array $row
     *
     * @return mixed
     */
    protected function formatCellValue($column, array $row)
    {
        if ($column == "checkbox") {
            $row[$column] = ilUtil::formCheckbox(false, 'set_id[]', $row['id']);
        }
        if ($column == "tstamp" || $column == "created") {
            return ilDatePresentation::formatDate(new ilDateTime($row[$column], IL_CAL_UNIX));
        }
        if ($column == "title_link") {
            $this->ctrl->setParameter($this->parent_obj, "set_id", $row['id']);
            if ($row["has_questions"]) {
                $href = $this->ctrl->getLinkTarget($this->parent_obj, "ilQuestionSetPoolPathsGUI.listPaths");
            } else {
                $href = $this->ctrl->getLinkTarget($this->parent_obj, "ilQuestionSetPoolSetQuestionsGUI.listQuestions");
            }

            $this->ctrl->setParameter($this->parent_obj, "set_id", "");
            $row[$column] = $href;
        }
        if ($column == 'actions') {
            $action = new ilAdvancedSelectionListGUI();
            $action->setId('asl_' . $row["id"]);
            $action->setListTitle($this->lng->txt('actions'));

            $this->ctrl->setParameter($this->parent_obj, "set_id", $row['id']);
            $action->addItem($this->plugin->txt("edit"), '', $this->ctrl->getLinkTarget($this->parent_obj, "ilQuestionSetPoolSetGUI.edit"));
            if ($row["has_questions"]) {
                $action->addItem($this->plugin->txt("show_added_questions"), '', $this->ctrl->getLinkTarget($this->parent_obj, "ilQuestionSetPoolSetQuestionsGUI.listQuestions"));
                $action->addItem($this->plugin->txt("path_overview"), '', $this->ctrl->getLinkTarget($this->parent_obj, "ilQuestionSetPoolPathGraphGUI.showPathOverview"));
            } else {
                $action->addItem($this->plugin->txt("add_questions"), '', $this->ctrl->getLinkTarget($this->parent_obj, "ilQuestionSetPoolSetQuestionsGUI.browseForQuestions"));
            }
            $action->addItem($this->lng->txt("delete"), '', $this->ctrl->getLinkTarget($this->parent_obj, "ilQuestionSetPoolSetsGUI.removeConfirmation"));
            $this->ctrl->setParameter($this->parent_obj, "set_id", "");

            $row["actions"] = $action->getHTML();
        }

        return parent::formatCellValue($column, $row);
    }
}
